@@ -104,7 +104,10 @@ int Style::integer(QString k) const {
 }
 
 QFont Style::font(QString k) const {
-  QFont f(string(k + "-family"));
+  QString fam = string(k + "-family");
+  if (fam=="Ubuntu")
+    fam = "UbuntuOrigWeight";
+  QFont f(fam);
   f.setPixelSize(int((100./72)*real(k + "-size")));
   f.setHintingPreference(QFont::PreferVerticalHinting);
   return f;
